@@ -38,11 +38,16 @@ module Data.Aeson
     , eitherDecode
     , eitherDecode'
     , encode
+    , encodeFile
     -- ** Variants for strict bytestrings
     , decodeStrict
+    , decodeFileStrict
     , decodeStrict'
+    , decodeFileStrict'
     , eitherDecodeStrict
+    , eitherDecodeFileStrict
     , eitherDecodeStrict'
+    , eitherDecodeFileStrict'
     -- * Core JSON types
     , Value(..)
     , Encoding
@@ -76,8 +81,8 @@ module Data.Aeson
     -- ** Generic JSON classes and options
     , GFromJSON(..)
     , FromArgs(..)
-    , GToJSON(..)
-    , GToEncoding(..)
+    , GToJSON
+    , GToEncoding
     , ToArgs(..)
     , Zero
     , One
@@ -87,7 +92,22 @@ module Data.Aeson
     , genericLiftToEncoding
     , genericParseJSON
     , genericLiftParseJSON
+    -- ** Generic and TH encoding configuration
+    , Options
     , defaultOptions
+    -- *** Options fields
+    -- $optionsFields
+    , fieldLabelModifier
+    , constructorTagModifier
+    , allNullaryToStringTag
+    , omitNothingFields
+    , sumEncoding
+    , unwrapUnaryRecords
+    , tagSingleConstructors
+    -- *** Options utilities
+    , SumEncoding(..)
+    , camelTo2
+    , defaultTaggedObject
 
     -- * Inspecting @'Value's@
     , withObject
@@ -96,6 +116,7 @@ module Data.Aeson
     , withNumber
     , withScientific
     , withBool
+    , withEmbeddedJSON
     -- * Constructors and accessors
     , Series
     , pairs
@@ -127,6 +148,10 @@ import qualified Data.ByteString.Lazy as L
 encode :: (ToJSON a) => a -> L.ByteString
 encode = encodingToLazyByteString . toEncoding
 
+-- | Efficiently serialize a JSON value as a lazy 'L.ByteString' and write it to a file.
+encodeFile :: (ToJSON a) => FilePath -> a -> IO ()
+encodeFile fp = L.writeFile fp . encode
+
 -- | Efficiently deserialize a JSON value from a lazy 'L.ByteString'.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
@@ -153,6 +178,18 @@ decodeStrict :: (FromJSON a) => B.ByteString -> Maybe a
 decodeStrict = decodeStrictWith jsonEOF fromJSON
 {-# INLINE decodeStrict #-}
 
+-- | Efficiently deserialize a JSON value from a file.
+-- If this fails due to incomplete or invalid input, 'Nothing' is
+-- returned.
+--
+-- The input file's content must consist solely of a JSON document,
+-- with no trailing data except for whitespace.
+--
+-- This function parses immediately, but defers conversion.  See
+-- 'json' for details.
+decodeFileStrict :: (FromJSON a) => FilePath -> IO (Maybe a)
+decodeFileStrict = fmap decodeStrict . B.readFile
+
 -- | Efficiently deserialize a JSON value from a lazy 'L.ByteString'.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
@@ -179,6 +216,18 @@ decodeStrict' :: (FromJSON a) => B.ByteString -> Maybe a
 decodeStrict' = decodeStrictWith jsonEOF' fromJSON
 {-# INLINE decodeStrict' #-}
 
+-- | Efficiently deserialize a JSON value from a file.
+-- If this fails due to incomplete or invalid input, 'Nothing' is
+-- returned.
+--
+-- The input file's content must consist solely of a JSON document,
+-- with no trailing data except for whitespace.
+--
+-- This function parses and performs conversion immediately.  See
+-- 'json'' for details.
+decodeFileStrict' :: (FromJSON a) => FilePath -> IO (Maybe a)
+decodeFileStrict' = fmap decodeStrict' . B.readFile
+
 eitherFormatError :: Either (JSONPath, String) a -> Either String a
 eitherFormatError = either (Left . uncurry formatError) Right
 {-# INLINE eitherFormatError #-}
@@ -194,6 +243,12 @@ eitherDecodeStrict =
   eitherFormatError . eitherDecodeStrictWith jsonEOF ifromJSON
 {-# INLINE eitherDecodeStrict #-}
 
+-- | Like 'decodeFileStrict' but returns an error message when decoding fails.
+eitherDecodeFileStrict :: (FromJSON a) => FilePath -> IO (Either String a)
+eitherDecodeFileStrict =
+  fmap (eitherFormatError . eitherDecodeStrictWith jsonEOF ifromJSON) . B.readFile
+{-# INLINE eitherDecodeFileStrict #-}
+
 -- | Like 'decode'' but returns an error message when decoding fails.
 eitherDecode' :: (FromJSON a) => L.ByteString -> Either String a
 eitherDecode' = eitherFormatError . eitherDecodeWith jsonEOF' ifromJSON
@@ -205,6 +260,12 @@ eitherDecodeStrict' =
   eitherFormatError . eitherDecodeStrictWith jsonEOF' ifromJSON
 {-# INLINE eitherDecodeStrict' #-}
 
+-- | Like 'decodeFileStrict'' but returns an error message when decoding fails.
+eitherDecodeFileStrict' :: (FromJSON a) => FilePath -> IO (Either String a)
+eitherDecodeFileStrict' =
+  fmap (eitherFormatError . eitherDecodeStrictWith jsonEOF' ifromJSON) . B.readFile
+{-# INLINE eitherDecodeFileStrict' #-}
+
 -- $use
 --
 -- This section contains basic information on the different ways to
@@ -366,7 +427,7 @@ eitherDecodeStrict' =
 -- The 'Object' type contains JSON objects:
 --
 -- > λ> decode "{\"name\":\"Dave\",\"age\":2}" :: Maybe Object
--- > Just (fromList) [("name",String "Dave"),("age",Number 2)]
+-- > Just (fromList [("name",String "Dave"),("age",Number 2)])
 --
 -- You can extract values from it with a parser using 'parse',
 -- 'parseEither' or, in this example, 'parseMaybe':
