@@ -36,6 +36,8 @@ module Data.Aeson.Types
     , ToJSON(..)
     , KeyValue(..)
     , modifyFailure
+    , parserThrowError
+    , parserCatchError
 
     -- ** Keys for maps
     , ToJSONKey(..)
@@ -63,8 +65,8 @@ module Data.Aeson.Types
     -- ** Generic JSON classes
     , GFromJSON(..)
     , FromArgs(..)
-    , GToJSON(..)
-    , GToEncoding(..)
+    , GToJSON
+    , GToEncoding
     , ToArgs(..)
     , Zero
     , One
@@ -82,6 +84,7 @@ module Data.Aeson.Types
     , withNumber
     , withScientific
     , withBool
+    , withEmbeddedJSON
 
     , pairs
     , foldable
@@ -102,7 +105,19 @@ module Data.Aeson.Types
     , listParser
 
     -- * Generic and TH encoding configuration
-    , Options(..)
+    , Options
+
+    -- ** Options fields
+    -- $optionsFields
+    , fieldLabelModifier
+    , constructorTagModifier
+    , allNullaryToStringTag
+    , omitNothingFields
+    , sumEncoding
+    , unwrapUnaryRecords
+    , tagSingleConstructors
+
+    -- ** Options utilities
     , SumEncoding(..)
     , camelTo
     , camelTo2
@@ -122,3 +137,6 @@ import Data.Foldable (toList)
 foldable :: (Foldable t, ToJSON a) => t a -> Encoding
 foldable = toEncoding . toList
 {-# INLINE foldable #-}
+
+-- $optionsFields
+-- The functions here are in fact record fields of the 'Options' type.
