@@ -55,6 +55,7 @@ module Data.Aeson.Types.FromJSON
     , withNumber
     , withScientific
     , withBool
+    , withEmbeddedJSON
 
     -- * Functions
     , fromJSON
@@ -82,6 +83,7 @@ import Prelude.Compat
 import Control.Applicative ((<|>), Const(..))
 import Control.Monad ((<=<), zipWithM)
 import Data.Aeson.Internal.Functions (mapKey)
+import Data.Aeson.Parser.Internal (eitherDecodeWith, jsonEOF)
 import Data.Aeson.Types.Generic
 import Data.Aeson.Types.Internal
 import Data.Attoparsec.Number (Number(..))
@@ -95,13 +97,13 @@ import Data.Hashable (Hashable(..))
 import Data.Int (Int16, Int32, Int64, Int8)
 import Data.List.NonEmpty (NonEmpty(..))
 import Data.Maybe (fromMaybe)
-import Data.Monoid ((<>))
+import Data.Semigroup ((<>))
 import Data.Proxy (Proxy(..))
 import Data.Ratio ((%), Ratio)
 import Data.Scientific (Scientific)
 import Data.Tagged (Tagged(..))
 import Data.Text (Text, pack, unpack)
-import Data.Time (Day, LocalTime, NominalDiffTime, TimeOfDay, UTCTime, ZonedTime)
+import Data.Time (Day, DiffTime, LocalTime, NominalDiffTime, TimeOfDay, UTCTime, ZonedTime)
 import Data.Time.Format (parseTime)
 import Data.Time.Locale.Compat (defaultTimeLocale)
 import Data.Traversable as Tr (sequence)
@@ -109,10 +111,12 @@ import Data.Vector (Vector)
 import Data.Version (Version, parseVersion)
 import Data.Word (Word16, Word32, Word64, Word8)
 import Foreign.Storable (Storable)
+import Foreign.C.Types (CTime (..))
 import GHC.Generics
 import Numeric.Natural (Natural)
 import Text.ParserCombinators.ReadP (readP_to_S)
 import Unsafe.Coerce (unsafeCoerce)
+import qualified Data.Aeson.Compat as Compat
 import qualified Data.Aeson.Parser.Time as Time
 import qualified Data.Attoparsec.ByteString.Char8 as A (endOfInput, parseOnly, scientific)
 import qualified Data.DList as DList
@@ -188,7 +192,7 @@ parseIntegralFromScientific :: forall a. Integral a => String -> Scientific -> P
 parseIntegralFromScientific expected s =
     case Scientific.floatingOrInteger s :: Either Double a of
         Right x -> pure x
-        Left _  -> fail $ "Floating number specified for " ++ expected ++ ": " ++ show s
+        Left _  -> fail $ "expected " ++ expected ++ ", encountered floating number " ++ show s
 {-# INLINE parseIntegralFromScientific #-}
 
 parseIntegral :: Integral a => String -> Value -> Parser a
@@ -315,7 +319,7 @@ genericLiftParseJSON opts pj pjl = fmap to1 . gParseJSON opts (From1Args pj pjl)
 --
 -- @
 -- instance 'FromJSON' Coord where
---     'parseJSON' = 'withObject' \"Coord\" $ \v -> Coord
+--     'parseJSON' = 'withObject' \"Coord\" $ \\v -> Coord
 --         '<$>' v '.:' \"x\"
 --         '<*>' v '.:' \"y\"
 -- @
@@ -664,6 +668,16 @@ withBool _        f (Bool arr) = f arr
 withBool expected _ v          = typeMismatch expected v
 {-# INLINE withBool #-}
 
+-- | Decode a nested JSON-encoded string.
+withEmbeddedJSON :: String -> (Value -> Parser a) -> Value -> Parser a
+withEmbeddedJSON _ innerParser (String txt) =
+    either fail innerParser $ eitherDecode (Compat.fromStrict $ T.encodeUtf8 txt)
+    where
+        eitherDecode = eitherFormatError . eitherDecodeWith jsonEOF ifromJSON
+        eitherFormatError = either (Left . uncurry formatError) Right
+withEmbeddedJSON name _ v = typeMismatch name v
+{-# INLINE withEmbeddedJSON #-}
+
 -- | Convert a value from JSON, failing if the types do not match.
 fromJSON :: (FromJSON a) => Value -> Result a
 fromJSON = parse parseJSON
@@ -791,6 +805,19 @@ instance GFromJSON arity U1 where
         | isEmptyArray v = pure U1
         | otherwise      = typeMismatch "unit constructor (U1)" v
 
+instance ( ConsFromJSON arity a
+         , AllNullary         (C1 c a) allNullary
+         , ParseSum     arity (C1 c a) allNullary
+         ) => GFromJSON arity (D1 d (C1 c a)) where
+    -- The option 'tagSingleConstructors' determines whether to wrap
+    -- a single-constructor type.
+    gParseJSON opts fargs
+        | tagSingleConstructors opts
+            = fmap M1
+            . (unTagged :: Tagged allNullary (Parser (C1 c a p)) -> Parser (C1 c a p))
+            . parseSum opts fargs
+        | otherwise = fmap M1 . fmap M1 . consParseJSON opts fargs
+
 instance (ConsFromJSON arity a) => GFromJSON arity (C1 c a) where
     -- Constructors need to be decoded differently depending on whether they're
     -- a record or not. This distinction is made by consParseJSON:
@@ -837,19 +864,19 @@ class ParseSum arity f allNullary where
     parseSum :: Options -> FromArgs arity a
              -> Value -> Tagged allNullary (Parser (f a))
 
-instance ( SumFromString           (a :+: b)
-         , FromPair          arity (a :+: b)
-         , FromTaggedObject  arity (a :+: b)
-         , FromUntaggedValue arity (a :+: b)
-         ) => ParseSum       arity (a :+: b) True where
+instance ( SumFromString           f
+         , FromPair          arity f
+         , FromTaggedObject  arity f
+         , FromUntaggedValue arity f
+         ) => ParseSum       arity f True where
     parseSum opts fargs
         | allNullaryToStringTag opts = Tagged . parseAllNullarySum    opts
         | otherwise                  = Tagged . parseNonAllNullarySum opts fargs
 
-instance ( FromPair          arity (a :+: b)
-         , FromTaggedObject  arity (a :+: b)
-         , FromUntaggedValue arity (a :+: b)
-         ) => ParseSum       arity (a :+: b) False where
+instance ( FromPair          arity f
+         , FromTaggedObject  arity f
+         , FromUntaggedValue arity f
+         ) => ParseSum       arity f False where
     parseSum opts fargs = Tagged . parseNonAllNullarySum opts fargs
 
 --------------------------------------------------------------------------------
@@ -875,11 +902,11 @@ instance (Constructor c) => SumFromString (C1 c U1) where
 
 --------------------------------------------------------------------------------
 
-parseNonAllNullarySum :: ( FromPair          arity (a :+: b)
-                         , FromTaggedObject  arity (a :+: b)
-                         , FromUntaggedValue arity (a :+: b)
+parseNonAllNullarySum :: ( FromPair          arity f
+                         , FromTaggedObject  arity f
+                         , FromUntaggedValue arity f
                          ) => Options -> FromArgs arity c
-                           -> Value -> Parser ((a :+: b) c)
+                           -> Value -> Parser (f c)
 parseNonAllNullarySum opts fargs =
     case sumEncoding opts of
       TaggedObject{..} ->
@@ -949,7 +976,7 @@ instance ( IsRecord                   f isRecord
 
 instance (FromRecord arity f) => FromTaggedObject'' arity f True where
     parseFromTaggedObject'' opts fargs _ =
-      Tagged . parseRecord opts fargs Nothing
+      Tagged . parseRecord opts fargs
 
 instance (GFromJSON arity f) => FromTaggedObject'' arity f False where
     parseFromTaggedObject'' opts fargs contentsFieldName = Tagged .
@@ -966,61 +993,64 @@ class ConsFromJSON arity f where
 
 class ConsFromJSON' arity f isRecord where
     consParseJSON' :: Options -> FromArgs arity a
-                   -> Maybe Text -- ^ A dummy label
-                                 --   (Nothing to use proper label)
                    -> Value -> Tagged isRecord (Parser (f a))
 
 instance ( IsRecord            f isRecord
          , ConsFromJSON' arity f isRecord
          ) => ConsFromJSON arity f where
-    consParseJSON opts fargs v = let
-      (v2,lab) = case (unwrapUnaryRecords opts,isUnary (undefined :: f a)) of
-                       -- use a dummy object with a dummy label
-        (True,True) -> (object [(pack "dummy",v)], Just $ pack "dummy")
-        _ ->(v,Nothing)
-      in (unTagged :: Tagged isRecord (Parser (f a)) -> Parser (f a))
-                       $ consParseJSON' opts fargs lab v2
+    consParseJSON opts fargs =
+      (unTagged :: Tagged isRecord (Parser (f a)) -> Parser (f a))
+        . consParseJSON' opts fargs
 
+instance OVERLAPPING_
+         ( GFromJSON arity a, FromRecord arity (S1 s a)
+         ) => ConsFromJSON' arity (S1 s a) True where
+    consParseJSON' opts fargs
+      | unwrapUnaryRecords opts = Tagged . gParseJSON opts fargs
+      | otherwise = Tagged . withObject "unary record" (parseRecord opts fargs)
 
-instance (FromRecord arity f) => ConsFromJSON' arity f True where
-    consParseJSON' opts fargs mlab = Tagged . withObject "record (:*:)"
-                                        (parseRecord opts fargs mlab)
+instance FromRecord arity f => ConsFromJSON' arity f True where
+    consParseJSON' opts fargs =
+      Tagged . withObject "record (:*:)" (parseRecord opts fargs)
 
-instance (GFromJSON arity f) => ConsFromJSON' arity f False where
-    consParseJSON' opts fargs _ = Tagged . gParseJSON opts fargs
+instance GFromJSON arity f => ConsFromJSON' arity f False where
+    consParseJSON' opts fargs = Tagged . gParseJSON opts fargs
 
 --------------------------------------------------------------------------------
 
 class FromRecord arity f where
     parseRecord :: Options -> FromArgs arity a
-                -> Maybe Text -- ^ A dummy label
-                              --   (Nothing to use proper label)
                 -> Object -> Parser (f a)
 
 instance ( FromRecord arity a
          , FromRecord arity b
          ) => FromRecord arity (a :*: b) where
-    parseRecord opts fargs _ obj =
-      (:*:) <$> parseRecord opts fargs Nothing obj
-            <*> parseRecord opts fargs Nothing obj
+    parseRecord opts fargs obj =
+      (:*:) <$> parseRecord opts fargs obj
+            <*> parseRecord opts fargs obj
 
 instance OVERLAPPABLE_ (Selector s, GFromJSON arity a) =>
   FromRecord arity (S1 s a) where
-    parseRecord opts fargs lab =
+    parseRecord opts fargs =
       (<?> Key label) . gParseJSON opts fargs <=< (.: label)
         where
-          label = fromMaybe defLabel lab
-          defLabel = pack . fieldLabelModifier opts $
-                       selName (undefined :: t s a p)
+          label = pack . fieldLabelModifier opts $ selName (undefined :: t s a p)
 
 instance INCOHERENT_ (Selector s, FromJSON a) =>
   FromRecord arity (S1 s (K1 i (Maybe a))) where
-    parseRecord _ _ (Just lab) obj = (M1 . K1) <$> obj .:? lab
-    parseRecord opts _ Nothing obj = (M1 . K1) <$> obj .:? pack label
+    parseRecord opts _ obj = M1 . K1 <$> obj .:? pack label
         where
           label = fieldLabelModifier opts $
                     selName (undefined :: t s (K1 i (Maybe a)) p)
 
+-- Parse an Option like a Maybe.
+instance INCOHERENT_ (Selector s, FromJSON a) =>
+  FromRecord arity (S1 s (K1 i (Semigroup.Option a))) where
+    parseRecord opts fargs obj = wrap <$> parseRecord opts fargs obj
+      where
+        wrap :: S1 s (K1 i (Maybe a)) p -> S1 s (K1 i (Semigroup.Option a)) p
+        wrap (M1 (K1 a)) = M1 (K1 (Semigroup.Option a))
+
 --------------------------------------------------------------------------------
 
 class FromProduct arity f where
@@ -1228,9 +1258,12 @@ instance FromJSONKey Float where
         _           -> Scientific.toRealFloat <$> parseScientificText t
 
 instance (FromJSON a, Integral a) => FromJSON (Ratio a) where
-    parseJSON = withObject "Rational" $ \obj ->
-                  (%) <$> obj .: "numerator"
-                      <*> obj .: "denominator"
+    parseJSON = withObject "Rational" $ \obj -> do
+        numerator <- obj .: "numerator"
+        denominator <- obj .: "denominator"
+        if denominator == 0
+        then fail "Ratio denominator was 0"
+        else pure $ numerator % denominator
     {-# INLINE parseJSON #-}
 
 -- | /WARNING:/ Only parse fixed-precision numbers from trusted input
@@ -1260,16 +1293,20 @@ instance FromJSONKey Integer where
     fromJSONKey = FromJSONKeyTextParser $ parseIntegralText "Integer"
 
 instance FromJSON Natural where
-    parseJSON = withScientific "Natural" $ \s ->
-      if Scientific.coefficient s < 0
-        then fail $ "Expected a Natural number but got the negative number: " <> show s
-        else pure $ truncate s
+    parseJSON value = do
+        integer :: Integer <- parseIntegral "Natural" value
+        if integer < 0 then
+            fail $ "expected Natural, encountered negative number " <> show integer
+        else
+            pure $ fromIntegral integer
 
 instance FromJSONKey Natural where
-    fromJSONKey = FromJSONKeyTextParser $ \t -> parseScientificText t >>= \s ->
-      if Scientific.coefficient s < 0
-        then fail $ "Expected a Natural number but got the negative number: " <> show s
-        else pure $ truncate s
+    fromJSONKey = FromJSONKeyTextParser $ \text -> do
+        integer :: Integer <- parseIntegralText "Natural" text
+        if integer < 0 then
+            fail $ "expected Natural, encountered negative number " <> show integer
+        else
+            pure $ fromIntegral integer
 
 instance FromJSON Int8 where
     parseJSON = parseBoundedIntegral "Int8"
@@ -1334,6 +1371,9 @@ instance FromJSON Word64 where
 instance FromJSONKey Word64 where
     fromJSONKey = FromJSONKeyTextParser $ parseBoundedIntegralText "Word64"
 
+instance FromJSON CTime where
+    parseJSON = fmap CTime . parseJSON
+    {-# INLINE parseJSON #-}
 
 instance FromJSON Text where
     parseJSON = withText "Text" pure
@@ -1526,8 +1566,8 @@ instance (FromJSONKey k, Ord k) => FromJSON1 (M.Map k) where
         FromJSONKeyTextParser f -> withObject "Map k v" $
             H.foldrWithKey (\k v m -> M.insert <$> f k <?> Key k <*> p v <?> Key k <*> m) (pure M.empty)
         FromJSONKeyValue f -> withArray "Map k v" $ \arr ->
-            M.fromList <$> (Tr.sequence .
-                zipWith (parseIndexedJSONPair f p) [0..] . V.toList $ arr)
+            fmap M.fromList . Tr.sequence .
+                zipWith (parseIndexedJSONPair f p) [0..] . V.toList $ arr
     {-# INLINE liftParseJSON #-}
 
 instance (FromJSONKey k, Ord k, FromJSON v) => FromJSON (M.Map k v) where
@@ -1606,8 +1646,8 @@ instance (FromJSONKey k, Eq k, Hashable k) => FromJSON1 (H.HashMap k) where
         FromJSONKeyTextParser f -> withObject "HashMap k v" $
             H.foldrWithKey (\k v m -> H.insert <$> f k <?> Key k <*> p v <?> Key k <*> m) (pure H.empty)
         FromJSONKeyValue f -> withArray "Map k v" $ \arr ->
-            H.fromList <$> (Tr.sequence .
-                zipWith (parseIndexedJSONPair f p) [0..] . V.toList $ arr)
+            fmap H.fromList . Tr.sequence .
+                zipWith (parseIndexedJSONPair f p) [0..] . V.toList $ arr
       where
         uc :: Parser (H.HashMap Text v) -> Parser (H.HashMap k v)
         uc = unsafeCoerce
@@ -1658,6 +1698,17 @@ instance FromJSONKey LocalTime where
     fromJSONKey = FromJSONKeyTextParser (Time.run Time.localTime)
 
 
+-- | Supported string formats:
+--
+-- @YYYY-MM-DD HH:MM Z@
+-- @YYYY-MM-DD HH:MM:SS Z@
+-- @YYYY-MM-DD HH:MM:SS.SSS Z@
+--
+-- The first space may instead be a @T@, and the second space is
+-- optional.  The @Z@ represents UTC.  The @Z@ may be replaced with a
+-- time zone offset of the form @+0000@ or @-08:00@, where the first
+-- two digits are hours, the @:@ is optional and the second two digits
+-- (also optional) are minutes.
 instance FromJSON ZonedTime where
     parseJSON = withText "ZonedTime" (Time.run Time.zonedTime)
 
@@ -1680,6 +1731,15 @@ instance FromJSON NominalDiffTime where
     parseJSON = withScientific "NominalDiffTime" $ pure . realToFrac
     {-# INLINE parseJSON #-}
 
+
+-- | /WARNING:/ Only parse lengths of time from trusted input
+-- since an attacker could easily fill up the memory of the target
+-- system by specifying a scientific number with a big exponent like
+-- @1e1000000000@.
+instance FromJSON DiffTime where
+    parseJSON = withScientific "DiffTime" $ pure . realToFrac
+    {-# INLINE parseJSON #-}
+
 -------------------------------------------------------------------------------
 -- base Monoid/Semigroup
 -------------------------------------------------------------------------------
