@@ -1,4 +1,3 @@
-{-# LANGUAGE PackageImports #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 module Main (main) where
@@ -14,7 +13,7 @@ import Data.Json.Builder
 import Twitter
 import Twitter.Manual ()
 import Typed.Common
-import qualified "aeson-benchmarks" Data.Aeson as Aeson
+import qualified Data.Aeson as Aeson
 import qualified Compare.JsonBench as JsonBench
 
 main :: IO ()
