@@ -11,8 +11,6 @@ import Prelude.Compat
 
 import Data.BufferBuilder.Json
 import Data.Int (Int64)
-import Data.Monoid ((<>))
-import Prelude hiding (id)
 import Twitter
 import qualified Data.BufferBuilder.Utf8 as UB
 
