@@ -6,11 +6,10 @@
 module Compare.JsonBuilder () where
 
 import Prelude ()
-import Prelude.Compat
+import Prelude.Compat hiding ((<>))
 
 import Data.Json.Builder
 import Data.Monoid ((<>))
-import Prelude hiding (id)
 import Twitter
 
 instance JsObject Metadata where
