@@ -12,6 +12,8 @@ import Criterion.Main
 import Data.Maybe (fromMaybe)
 import qualified Data.Aeson as A
 import qualified Data.Aeson.Text as A
+import qualified Data.Aeson.Parser.Internal as I
+import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.Text.Lazy          as TL
 import qualified Data.Text.Lazy.Builder  as TLB
@@ -35,11 +37,18 @@ decodeJ s =
     J.Ok v -> v
     J.Error _ -> error "fail to parse via JSON"
 
-decodeA :: BL.ByteString -> A.Value
-decodeA s = fromMaybe (error "fail to parse via Aeson") $ A.decode s
+decode :: BL.ByteString -> A.Value
+decode s = fromMaybe (error "fail to parse via Aeson") $ A.decode s
 
-decodeA' :: BL.ByteString -> A.Value
-decodeA' s = fromMaybe (error "fail to parse via Aeson") $ A.decode' s
+decode' :: BL.ByteString -> A.Value
+decode' s = fromMaybe (error "fail to parse via Aeson") $ A.decode' s
+
+decodeS :: BS.ByteString -> A.Value
+decodeS s = fromMaybe (error "fail to parse via Aeson") $ A.decodeStrict' s
+
+decodeIP :: BL.ByteString -> A.Value
+decodeIP s = fromMaybe (error "fail to parse via Parser.decodeWith") $
+    I.decodeWith I.jsonEOF A.fromJSON s
 
 encodeJ :: J.JSValue -> BL.ByteString
 encodeJ = toLazyByteString . fromString . J.encode
@@ -55,32 +64,37 @@ main = do
   let enFile = "json-data/twitter100.json"
       jpFile = "json-data/jp100.json"
   enA <- BL.readFile enFile
+  enS <- BS.readFile enFile
   enJ <- readFile enFile
   jpA <- BL.readFile jpFile
+  jpS <- BS.readFile jpFile
   jpJ <- readFile jpFile
   defaultMain [
       bgroup "decode" [
         bgroup "en" [
-          bench "aeson/lazy" $ nf decodeA enA
-        , bench "aeson/strict" $ nf decodeA' enA
-        , bench "json"  $ nf decodeJ enJ
+          bench "aeson/lazy"     $ nf decode enA
+        , bench "aeson/strict"   $ nf decode' enA
+        , bench "aeson/stricter" $ nf decodeS enS
+        , bench "aeson/parser"   $ nf decodeIP enA
+        , bench "json"           $ nf decodeJ enJ
         ]
       , bgroup "jp" [
-          bench "aeson" $ nf decodeA jpA
-        , bench "json"  $ nf decodeJ jpJ
+          bench "aeson"          $ nf decode jpA
+        , bench "aeson/stricter" $ nf decodeS jpS
+        , bench "json"           $ nf decodeJ jpJ
         ]
       ]
     , bgroup "encode" [
         bgroup "en" [
-          bench "aeson-to-bytestring" $ nf A.encode (decodeA enA)
-        , bench "aeson-via-text-to-bytestring" $ nf encodeViaText (decodeA enA)
-        , bench "aeson-to-text" $ nf encodeToText (decodeA enA)
+          bench "aeson-to-bytestring" $ nf A.encode (decode enA)
+        , bench "aeson-via-text-to-bytestring" $ nf encodeViaText (decode enA)
+        , bench "aeson-to-text" $ nf encodeToText (decode enA)
         , bench "json"  $ nf encodeJ (decodeJ enJ)
         ]
       , bgroup "jp" [
-          bench "aeson-to-bytestring" $ nf A.encode (decodeA jpA)
-        , bench "aeson-via-text-to-bytestring" $ nf encodeViaText (decodeA jpA)
-        , bench "aeson-to-text" $ nf encodeToText (decodeA jpA)
+          bench "aeson-to-bytestring" $ nf A.encode (decode jpA)
+        , bench "aeson-via-text-to-bytestring" $ nf encodeViaText (decode jpA)
+        , bench "aeson-to-text" $ nf encodeToText (decode jpA)
         , bench "json"  $ nf encodeJ (decodeJ jpJ)
         ]
       ]
