@@ -6,17 +6,19 @@
 -- automatically derive the Generic class for us.
 
 {-# LANGUAGE DeriveGeneric #-}
-
 {-# LANGUAGE OverloadedStrings #-}
 
+module Main (main) where
+
+import Prelude ()
+import Prelude.Compat
+
 import Data.Aeson (FromJSON, ToJSON, decode, encode)
 import qualified Data.ByteString.Lazy.Char8 as BL
+import GHC.Generics (Generic)
 
 -- To decode or encode a value using the generic machinery, we must
 -- make the type an instance of the Generic class.
-
-import GHC.Generics (Generic)
-
 data Coord = Coord { x :: Double, y :: Double }
              deriving (Show, Generic)
 
@@ -31,5 +33,5 @@ main :: IO ()
 main = do
   let req = decode "{\"x\":3.0,\"y\":-1.0}" :: Maybe Coord
   print req
-  let reply = Coord 123.4 20
+  let reply = Coord { x = 123.4, y = 20 }
   BL.putStrLn (encode reply)
