@@ -1,10 +1,13 @@
 -- We can use Template Haskell (TH) to generate instances of the
 -- FromJSON and ToJSON classes automatically.  This is the fastest way
 -- to add JSON support for a type.
-
+{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
 
-{-# LANGUAGE OverloadedStrings #-}
+module Main (main) where
+
+import Prelude ()
+import Prelude.Compat
 
 import Data.Aeson (decode, encode)
 import Data.Aeson.TH (deriveJSON, defaultOptions)
@@ -21,5 +24,5 @@ main :: IO ()
 main = do
   let req = decode "{\"x\":3.0,\"y\":-1.0}" :: Maybe Coord
   print req
-  let reply = Coord 123.4 20
+  let reply = Coord { x = 123.4, y = 20 }
   BL.putStrLn (encode reply)
