@@ -8,21 +8,26 @@
 -- For Data.Aeson.Types.camelTo
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 
+#if MIN_VERSION_base(4,9,0)
+{-# OPTIONS_GHC -fno-warn-unused-top-binds #-}
+#endif
+
 module UnitTests
     (
       ioTests
     , tests
+    , withEmbeddedJSONTest
     ) where
 
 import Prelude ()
 import Prelude.Compat
 
 import Control.Monad (forM, forM_)
-import Data.Aeson ((.=), (.:), (.:?), (.:!), FromJSON(..), FromJSONKeyFunction(..), FromJSONKey(..), ToJSON1(..), decode, eitherDecode, encode, genericToEncoding, genericToJSON, object, withObject)
+import Data.Aeson ((.=), (.:), (.:?), (.:!), FromJSON(..), FromJSONKeyFunction(..), FromJSONKey(..), ToJSON1(..), decode, eitherDecode, encode, fromJSON, genericParseJSON, genericToEncoding, genericToJSON, object, withObject, withEmbeddedJSON)
 import Data.Aeson.Internal (JSONPathElement(..), formatError)
 import Data.Aeson.TH (deriveJSON, deriveToJSON, deriveToJSON1)
 import Data.Aeson.Text (encodeToTextBuilder)
-import Data.Aeson.Types (Options(..), ToJSON(..), Value, camelTo, camelTo2, defaultOptions, omitNothingFields)
+import Data.Aeson.Types (Options(..), Result(Success), ToJSON(..), Value(Null), camelTo, camelTo2, defaultOptions, omitNothingFields, parse)
 import Data.Char (toUpper)
 import Data.Either.Compat (isLeft, isRight)
 import Data.Hashable (hash)
@@ -38,8 +43,8 @@ import GHC.Generics (Generic)
 import Instances ()
 import System.Directory (getDirectoryContents)
 import System.FilePath ((</>), takeExtension, takeFileName)
-import Test.Framework (Test, testGroup)
-import Test.Framework.Providers.HUnit (testCase)
+import Test.Tasty (TestTree, testGroup)
+import Test.Tasty.HUnit (testCase)
 import Test.HUnit (Assertion, assertBool, assertFailure, assertEqual)
 import Text.Printf (printf)
 import UnitTests.NullaryConstructors (nullaryConstructors)
@@ -57,7 +62,7 @@ import qualified SerializationFormatSpec
 import Data.Aeson.Parser.UnescapeFFI ()
 import Data.Aeson.Parser.UnescapePure ()
 
-tests :: Test
+tests :: TestTree
 tests = testGroup "unit" [
     testGroup "SerializationFormatSpec" SerializationFormatSpec.tests
   , testGroup "ErrorMessages" ErrorMessages.tests
@@ -92,6 +97,10 @@ tests = testGroup "unit" [
   , testCase "PR #455" pr455
   , testCase "Unescape string (PR #477)" unescapeString
   , testCase "Show Options" showOptions
+  , testGroup "SingleMaybeField" singleMaybeField
+  , testCase "withEmbeddedJSON" withEmbeddedJSONTest
+  , testCase "SingleFieldCon" singleFieldCon
+  , testCase "Ratio with denominator 0" ratioDenominator0
   ]
 
 roundTripCamel :: String -> Assertion
@@ -338,13 +347,13 @@ issue351 = [
 -- Comparison between bytestring and text encoders
 ------------------------------------------------------------------------------
 
-ioTests :: IO [Test]
+ioTests :: IO [TestTree]
 ioTests = do
   enc <- encoderComparisonTests
   js <- jsonTestSuite
   return [enc, js]
 
-encoderComparisonTests :: IO Test
+encoderComparisonTests :: IO TestTree
 encoderComparisonTests = do
   encoderTests <- forM testFiles $ \file0 -> do
       let file = "benchmarks/json-data/" ++ file0
@@ -404,7 +413,7 @@ unescapeString = do
 
 -- JSONTestSuite
 
-jsonTestSuiteTest :: FilePath -> Test
+jsonTestSuiteTest :: FilePath -> TestTree
 jsonTestSuiteTest path = testCase fileName $ do
     payload <- L.readFile path
     let result = eitherDecode payload :: Either String Value
@@ -419,7 +428,7 @@ jsonTestSuiteTest path = testCase fileName $ do
 -- Build a collection of tests based on the current contents of the
 -- JSONTestSuite test directories.
 
-jsonTestSuite :: IO Test
+jsonTestSuite :: IO TestTree
 jsonTestSuite = do
   let suitePath = "tests/JSONTestSuite"
   let suites = ["test_parsing", "test_transform"]
@@ -493,10 +502,60 @@ showOptions =
         ++ ", omitNothingFields = False"
         ++ ", sumEncoding = TaggedObject {tagFieldName = \"tag\", contentsFieldName = \"contents\"}"
         ++ ", unwrapUnaryRecords = False"
+        ++ ", tagSingleConstructors = False"
         ++ "}")
         (show defaultOptions)
 
+newtype SingleMaybeField = SingleMaybeField { smf :: Maybe Int }
+  deriving (Eq, Show, Generic)
+
+singleMaybeField :: [TestTree]
+singleMaybeField = do
+  (gName, gToJSON, gToEncoding, gFromJSON) <-
+    [ ("generic", genericToJSON opts, genericToEncoding opts, parse (genericParseJSON opts))
+    , ("th", toJSON, toEncoding, fromJSON) ]
+  return $
+    testCase gName $ do
+      assertEqual "toJSON"     Null (gToJSON v)
+      assertEqual "toEncoding" (toEncoding (gToJSON v)) (gToEncoding v)
+      assertEqual "fromJSON"   (Success v) (gFromJSON Null)
+  where
+    v = SingleMaybeField Nothing
+    opts = defaultOptions{omitNothingFields=True,unwrapUnaryRecords=True}
+
+
+newtype EmbeddedJSONTest = EmbeddedJSONTest Int
+  deriving (Eq, Show)
+
+instance FromJSON EmbeddedJSONTest where
+  parseJSON =
+    withObject "Object" $ \o ->
+      EmbeddedJSONTest <$> (o .: "prop" >>= withEmbeddedJSON "Quoted Int" parseJSON)
+
+withEmbeddedJSONTest :: Assertion
+withEmbeddedJSONTest =
+  assertEqual "Unquote embedded JSON" (Right $ EmbeddedJSONTest 1) (eitherDecode "{\"prop\":\"1\"}")
+
+-- Regression test for https://github.com/bos/aeson/issues/627
+newtype SingleFieldCon = SingleFieldCon Int deriving (Eq, Show, Generic)
+
+instance FromJSON SingleFieldCon where
+  parseJSON = genericParseJSON defaultOptions{unwrapUnaryRecords=True}
+  -- This option should have no effect on this type
+
+singleFieldCon :: Assertion
+singleFieldCon =
+  assertEqual "fromJSON" (Right (SingleFieldCon 0)) (eitherDecode "0")
+
+ratioDenominator0 :: Assertion
+ratioDenominator0 =
+  assertEqual "Ratio with denominator 0"
+    (Left "Error in $: Ratio denominator was 0")
+    (eitherDecode "{ \"numerator\": 1, \"denominator\": 0 }" :: Either String Rational)
+
 deriveJSON defaultOptions{omitNothingFields=True} ''MyRecord
 
 deriveToJSON  defaultOptions ''Foo
 deriveToJSON1 defaultOptions ''Foo
+
+deriveJSON defaultOptions{omitNothingFields=True,unwrapUnaryRecords=True} ''SingleMaybeField
